/*
 *
 * Copyright (c) 2007, Palmer Performance Engineering, Inc. All Rights Reserved.
 * 
 * This sample source code ("Software") is furnished under license and may only be
 * used or copied in accordance with the terms of that license. No license, express
 * or implied, by estoppel or otherwise, to any intellectual property rights is
 * granted by this document. The Software is subject to change without notice, and
 * should not be construed as a commitment by Palmer Performance Engineering, Inc.
 * to market, license, sell or support any product or technology. Unless otherwise
 * provided for in the license under which this Software is provided, the Software is
 * provided AS IS, with no warranties of any kind, express or implied. Except as expressly
 * permitted by the Software license, neither Palmer Performance Engineering, Inc. nor
 * its suppliers assumes any responsibility or liability for any errors or inaccuracies
 * that may appear herein. Except as expressly permitted by the Software license, no part
 * of the Software may be reproduced, stored in a retrieval system, transmitted in
 * any form, or distributed by any means without the express written consent of
 * Palmer Performance Engineering, Inc.
 *
 */

#ifndef __PLUGINAPI_H
#define __PLUGINAPI_H


#define PLUGIN_API				__stdcall

typedef unsigned long			PLUGIN_STATUS;

typedef enum
{
	PLUGIN_INVALID_VERSION	= 0,
	PLUGIN_VERSION_1		= 1,
} PLUGIN_API_VERSION;


/////////////////////////////////////////////////////////
// Plugin Status Codes
/////////////////////////////////////////////////////////
#define PLUGIN_STATUS_SUCCESS						0
#define PLUGIN_STATUS_ERROR							1		// General plugin error - will call Plugin_GetLastErrorDescription() for details
#define PLUGIN_STATUS_PID_NOT_SUPPORTED				2
#define PLUGIN_STATUS_BUFFER_TOO_SMALL				3


#define PLUGIN_PID_VALUETYPE_NUMBER					1
#define PLUGIN_PID_VALUETYPE_STRING					2

#pragma pack(push, 4)

typedef struct
{
	unsigned long	PidValueType;					// Parameter value type
	unsigned long	TimeStamp;						// The time of the sample, in milliseconds since logging started
	double			EnglishValue;					// For PLUGIN_PID_VALUETYPE_NUMBER types
	double			MetricValue;					// For PLUGIN_PID_VALUETYPE_NUMBER types
	char			EnglishValueString[128];		// For PLUGIN_PID_VALUETYPE_STRING types
	char			MetricValueString[128];			// For PLUGIN_PID_VALUETYPE_STRING types
} PLUGIN_PID_VALUE;

typedef struct
{
	char			UniqueId[64];					// Parameter Id - i.e. "SAE.RPM"
	char			Name[256];						// Parameter name i.e. Engine RPM
	char			Category[64];					// Parameter category i.e. Misfire
	char			Manufacturer[64];				// Parameter manufacturer i.e. Palmer Performance Engineering, Inc.
	char			EnglishUnits[32];				// English units i.e. MPH or RPM
	char			MetricUnits[32];				// Metric units i.e. KMH or RPM
	unsigned long	EnglishPrecision;				// Number of decimal places to show for English units
	unsigned long	MetricPrecision;				// Number of decimal places to show for Metric units
} PLUGIN_PID_INFO;

#pragma pack(pop)



typedef void* (__stdcall *GetCallbackFuncPtr)(void* pInstance, unsigned long CallbackNumber);


#define PLUGIN_CALLBACK_SET_SETTING_VALUE_STRING		1
#define PLUGIN_CALLBACK_SET_SETTING_VALUE_NUMBER		2
#define PLUGIN_CALLBACK_GET_SETTING_VALUE_STRING		3
#define PLUGIN_CALLBACK_GET_SETTING_VALUE_NUMBER		4
#define PLUGIN_CALLBACK_PRINT_CONSOLE_TEXT				5


typedef PLUGIN_STATUS (__stdcall *PluginCallback_SetSettingValueStringPtr)(void* pInstance, const char* PluginId, const char* SettingId, const char* SettingValue);
typedef PLUGIN_STATUS (__stdcall *PluginCallback_SetSettingValueNumberPtr)(void* pInstance, const char* PluginId, const char* SettingId, double SettingValue);
typedef PLUGIN_STATUS (__stdcall *PluginCallback_GetSettingValueStringPtr)(void* pInstance, const char* PluginId, const char* SettingId, char* SettingValue, int SettingValueSize);
typedef PLUGIN_STATUS (__stdcall *PluginCallback_GetSettingValueNumberPtr)(void* pInstance, const char* PluginId, const char* SettingId, double* SettingValue);
typedef PLUGIN_STATUS (__stdcall *PluginCallback_PrintConsoleTextPtr)(const char* Text);






PLUGIN_API_VERSION PLUGIN_API	Plugin_GetPluginApiVersion(void);

PLUGIN_STATUS PLUGIN_API		Plugin_Initialize(void* pInstance, PLUGIN_API_VERSION CurrentApiVersion, GetCallbackFuncPtr pGetCallbackFunc);
PLUGIN_STATUS PLUGIN_API		Plugin_Uninitialize(void);

const char* PLUGIN_API			Plugin_GetPluginName(void);

const char* PLUGIN_API			Plugin_GetLastErrorDescription(void);

PLUGIN_STATUS PLUGIN_API		Plugin_DisplaySettingsDialog(void);

unsigned long PLUGIN_API		Plugin_GetPidCount(void);
PLUGIN_STATUS PLUGIN_API		Plugin_GetPidInfo(unsigned long PidIndex, PLUGIN_PID_INFO* PidInfo);
PLUGIN_STATUS PLUGIN_API		Plugin_IsPidSupported(const char* PidUniqueId);

PLUGIN_STATUS PLUGIN_API		Plugin_OpenConnection(void);
PLUGIN_STATUS PLUGIN_API		Plugin_CloseConnection(void);

PLUGIN_STATUS PLUGIN_API		Plugin_Worker(void);

PLUGIN_STATUS PLUGIN_API		Plugin_LoggingStart(int SelectedPidCount, char** SelectedPidList);
PLUGIN_STATUS PLUGIN_API		Plugin_LoggingStop(void);
PLUGIN_STATUS PLUGIN_API		Plugin_LoggingPause(void);
PLUGIN_STATUS PLUGIN_API		Plugin_LoggingResume(void);
PLUGIN_STATUS PLUGIN_API		Plugin_LoggingGetPidValue(const char* PidUniqueId, PLUGIN_PID_VALUE* PidValue);



#endif // defined __PLUGINAPI_H
