/*
 *
 * Copyright (c) 2007, Palmer Performance Engineering, Inc. All Rights Reserved.
 * 
 * This sample source code ("Software") is furnished under license and may only be
 * used or copied in accordance with the terms of that license. No license, express
 * or implied, by estoppel or otherwise, to any intellectual property rights is
 * granted by this document. The Software is subject to change without notice, and
 * should not be construed as a commitment by Palmer Performance Engineering, Inc.
 * to market, license, sell or support any product or technology. Unless otherwise
 * provided for in the license under which this Software is provided, the Software is
 * provided AS IS, with no warranties of any kind, express or implied. Except as expressly
 * permitted by the Software license, neither Palmer Performance Engineering, Inc. nor
 * its suppliers assumes any responsibility or liability for any errors or inaccuracies
 * that may appear herein. Except as expressly permitted by the Software license, no part
 * of the Software may be reproduced, stored in a retrieval system, transmitted in
 * any form, or distributed by any means without the express written consent of
 * Palmer Performance Engineering, Inc.
 *
 */

#include <windows.h>
#include <assert.h>

#include "pluginapi.h"


void*								g_pObdiiInstance = NULL;
PLUGIN_API_VERSION					g_SuppliedApiVersion = PLUGIN_VERSION_1;
GetCallbackFuncPtr					g_pGetCallbackFunc = NULL;
DWORD								g_LoggingStartTime = 0;

PluginCallback_PrintConsoleTextPtr	g_pPrintConsoleText = NULL;


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       void* lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		break;
	case DLL_THREAD_ATTACH:
		break;
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		break;
    }

    return TRUE;
}

PLUGIN_API_VERSION PLUGIN_API Plugin_GetPluginApiVersion(void)
{
	return PLUGIN_VERSION_1;
}

PLUGIN_STATUS PLUGIN_API Plugin_Initialize(void* pInstance, PLUGIN_API_VERSION CurrentApiVersion, GetCallbackFuncPtr pGetCallbackFunc)
{
	g_pObdiiInstance = pInstance;
	g_SuppliedApiVersion = CurrentApiVersion;
	g_pGetCallbackFunc = pGetCallbackFunc;

	g_pPrintConsoleText = (PluginCallback_PrintConsoleTextPtr)g_pGetCallbackFunc(g_pObdiiInstance, PLUGIN_CALLBACK_PRINT_CONSOLE_TEXT);

	g_pPrintConsoleText("PPE Plugin Demo Initialized!");

	return PLUGIN_STATUS_SUCCESS;
}

PLUGIN_STATUS PLUGIN_API Plugin_Uninitialize(void)
{
	return PLUGIN_STATUS_SUCCESS;
}

const char* PLUGIN_API Plugin_GetPluginName(void)
{
	return "Palmer Performance OBD-II Plugin Demo";
}

const char* PLUGIN_API Plugin_GetLastErrorDescription(void)
{
	return "Last Error Description Text Goes Here";
}

PLUGIN_STATUS PLUGIN_API Plugin_DisplaySettingsDialog(void)
{
	// The plugin demo does not support a settings dialog.
	// If it did support a settings dialog, you would use
	// the callbacks to get/set setting values.
	MessageBox(NULL, "PPE Plugin Demo Settings Dialog", "Plugin Settings", MB_OK);
	return PLUGIN_STATUS_ERROR;
}

unsigned long PLUGIN_API Plugin_GetPidCount(void)
{
	// The plugin demo supports 2 PIDs
	return 2;
}

PLUGIN_STATUS PLUGIN_API Plugin_GetPidInfo(unsigned long PidIndex, PLUGIN_PID_INFO* PidInfo)
{
	if (PidIndex == 0)
	{
		strcpy(PidInfo->UniqueId, "PLUGIN.PID1");
		strcpy(PidInfo->Name, "Palmer Performance OBD-II Plugin Demo PID One");
		strcpy(PidInfo->Category, "Misc");
		strcpy(PidInfo->Manufacturer, "Palmer Performance Engineering, Inc.");
		strcpy(PidInfo->EnglishUnits, "mph");
		strcpy(PidInfo->MetricUnits, "km/h");
		PidInfo->EnglishPrecision = 0;
		PidInfo->MetricPrecision = 0;

		return PLUGIN_STATUS_SUCCESS;
	}
	else if (PidIndex == 1)
	{
		strcpy(PidInfo->UniqueId, "PLUGIN.PID2");
		strcpy(PidInfo->Name, "Palmer Performance OBD-II Plugin Demo PID Two");
		strcpy(PidInfo->Category, "Misc");
		strcpy(PidInfo->Manufacturer, "Palmer Performance Engineering, Inc.");
		strcpy(PidInfo->EnglishUnits, "rpm");
		strcpy(PidInfo->MetricUnits, "rpm");
		PidInfo->EnglishPrecision = 0;
		PidInfo->MetricPrecision = 0;

		return PLUGIN_STATUS_SUCCESS;
	}

	// Invalid PID index
	return PLUGIN_STATUS_ERROR;
}

PLUGIN_STATUS PLUGIN_API Plugin_IsPidSupported(const char* PidUniqueId)
{
	if (stricmp(PidUniqueId, "PLUGIN.PID1") == 0)
	{
		// PID1 is not supported right now
		return PLUGIN_STATUS_PID_NOT_SUPPORTED;
	}
	else if (stricmp(PidUniqueId, "PLUGIN.PID2") == 0)
	{
		// PID2 is supported
		return PLUGIN_STATUS_SUCCESS;
	}

	// Invalid PID
	return PLUGIN_STATUS_ERROR;
}

PLUGIN_STATUS PLUGIN_API Plugin_OpenConnection(void)
{
	// The OBD-II connection has just been opened, so
	// establish the connection to the plugin device
	return PLUGIN_STATUS_SUCCESS;
}

PLUGIN_STATUS PLUGIN_API Plugin_CloseConnection(void)
{
	// Close the connection with the plugin device
	return PLUGIN_STATUS_SUCCESS;
}

PLUGIN_STATUS PLUGIN_API Plugin_Worker(void)
{
	// The OBD-II scantool will call this funtion repeatedly
	// in a loop. This function must return immediately.
	// If it needs to block for any reason, the plugin
	// should spawn a worker thread to take care of it.
	return PLUGIN_STATUS_SUCCESS;
}

PLUGIN_STATUS PLUGIN_API Plugin_LoggingStart(int SelectedPidCount, char** SelectedPidList)
{
	// OBD-II data logging has just been started
	// SelectedPidCount and SelectedPidList inform us
	// if any of this plugins PIDs are selected for
	// the data logging.

	g_LoggingStartTime = GetTickCount();

	return PLUGIN_STATUS_SUCCESS;
}

PLUGIN_STATUS PLUGIN_API Plugin_LoggingStop(void)
{
	// OBD-II data logging has just been stopped.
	return PLUGIN_STATUS_SUCCESS;
}

PLUGIN_STATUS PLUGIN_API Plugin_LoggingPause(void)
{
	// OBD-II data logging has just been paused.
	return PLUGIN_STATUS_SUCCESS;
}

PLUGIN_STATUS PLUGIN_API Plugin_LoggingResume(void)
{
	// OBD-II data logging has just been resumed.
	return PLUGIN_STATUS_SUCCESS;
}

PLUGIN_STATUS PLUGIN_API Plugin_LoggingGetPidValue(const char* PidUniqueId, PLUGIN_PID_VALUE* PidValue)
{
	if (stricmp(PidUniqueId, "PLUGIN.PID1") == 0)
	{
		// PID1 is not supported right now and should not be getting data logged.
		assert(false);
		return PLUGIN_STATUS_PID_NOT_SUPPORTED;
	}
	else if (stricmp(PidUniqueId, "PLUGIN.PID2") == 0)
	{
		// PID2 is supported, report the value
		PidValue->PidValueType = PLUGIN_PID_VALUETYPE_NUMBER;
		PidValue->TimeStamp = GetTickCount() - g_LoggingStartTime;
		PidValue->EnglishValue = 12345;
		PidValue->MetricValue = 67890;
		return PLUGIN_STATUS_SUCCESS;
	}

	// Invalid PID
	return PLUGIN_STATUS_ERROR;
}
